#include <iom8.h> 
#include "intrinsics.h"
#include <math.h>
#include <stdlib.h>

//============================================================
// Paskirtis: ADC rezultato gavimas
//
// Inputs: Kanalo pasirinkimas
//
// Outputs: ADC reiksme
//
//============================================================
adc_convert(int channel){
  ADCSR |= (1 << ADPS2) | (0 << ADPS1) | (1 << ADPS0);     // //Set ADC prescaler to 128 - 125KHz sample rate @ 16MHz
  ADCSR |= (1 << ADEN);                                    // Enable ADC
  ADMUX=0;  
  ADMUX |= (1 << REFS0)|(1 << REFS1);                                    // Set ADC reference to AVCC 
  ADMUX |=  channel;                          // Channel selection
  ADCSR |= (1 << ADSC);                                    // Start A2D Conversions
  while(!((ADCSR>>ADIF)&1)){}                              // wait till converted
  return(ADC);
}
//============================================================
// Paskirtis: ADC rezultato vidurkinimas
//
// Inputs: Kanalo pasirinkimas ir vidurkinimo sk.
//
// Outputs: Suvidurkinta ADC reiksme
//
//============================================================
adc_average(int channel, int averaging){
  unsigned int sum,i;  
  for(i =1; i < averaging;i++){
    sum=sum+adc_convert(channel);
  } 
  return(sum/averaging);
}
//============================================================
// Paskirtis: 7 segment LED 
//
// Inputs: Simbolis ir tasko ijungimas
//
// Outputs: Simbolio parodymas ant ekranelio
//
//============================================================
seven_segment(unsigned char symbol, unsigned char dot){
DDRD=(1<<2)|(1<<3)| (1<<4)| (1<<5)| (1<<6)| (1<<7);//set as outputs
DDRC=(1<<4)| (1<<5);                               //set as outputs


#define CHAR_SPACE          0
#define CHAR_ALL            (SEG_a|SEG_b|SEG_c|SEG_d|SEG_e|SEG_f|SEG_g|SEG_h)
#define CHAR_0              (SEG_a|SEG_b|SEG_c|SEG_d|SEG_e|SEG_f)
#define CHAR_1              (SEG_b|SEG_c)
#define CHAR_2              (SEG_a|SEG_b|SEG_d|SEG_e|SEG_g)
#define CHAR_3              (SEG_a|SEG_b|SEG_c|SEG_d|SEG_g)
#define CHAR_4              (SEG_b|SEG_c|SEG_f|SEG_g)
#define CHAR_5              (SEG_a|SEG_c|SEG_d|SEG_f|SEG_g)
#define CHAR_6              (SEG_a|SEG_c|SEG_d|SEG_e|SEG_f|SEG_g)
#define CHAR_7              (SEG_a|SEG_b|SEG_c)
#define CHAR_8              (SEG_a|SEG_b|SEG_c|SEG_d|SEG_e|SEG_f|SEG_g)
#define CHAR_9              (SEG_a|SEG_b|SEG_c|SEG_d|SEG_f|SEG_g)
#define CHAR_A              (SEG_a|SEG_b|SEG_c|SEG_e|SEG_f|SEG_g)
#define CHAR_B              (SEG_c|SEG_d|SEG_e|SEG_f|SEG_g)
#define CHAR_C              (SEG_a|SEG_d|SEG_e|SEG_f)
#define CHAR_D              (SEG_b|SEG_c|SEG_d|SEG_e|SEG_g)
#define CHAR_E              (SEG_a|SEG_d|SEG_e|SEG_f|SEG_g)
#define CHAR_F              (SEG_a|SEG_e|SEG_f|SEG_g)
#define CHAR_G              (SEG_a|SEG_c|SEG_d|SEG_e|SEG_f)
#define CHAR_L              (SEG_d|SEG_e|SEG_f)
#define CHAR_P              (SEG_a|SEG_b|SEG_e|SEG_f|SEG_g)
#define CHAR_U              (SEG_b|SEG_c|SEG_d|SEG_e|SEG_f)
#define CHAR_H              (SEG_b|SEG_c|SEG_e|SEG_f|SEG_g)
#define CHAR_N              (SEG_c|SEG_e|SEG_g)
#define CHAR_MINUS          (SEG_g)

#define SEG_a       (1<<6)
#define SEG_b       (1<<7)
#define SEG_c       (1<<2)
#define SEG_d       (1<<0)
#define SEG_e       (1<<3)
#define SEG_f       (1<<5)
#define SEG_g       (1<<4)
#define SEG_h       (1<<1)

const unsigned char led_table[19] =
{
    CHAR_0,
    CHAR_1,
    CHAR_2,
    CHAR_3,
    CHAR_4,
    CHAR_5,
    CHAR_6,
    CHAR_7,
    CHAR_8,
    CHAR_9,
    CHAR_A,
    CHAR_B,
    CHAR_C,
    CHAR_D,
    CHAR_E,
    CHAR_F,
    CHAR_MINUS,
    CHAR_ALL,
    CHAR_SPACE  
};


PORTD=(led_table[symbol]&252);
PORTC=((led_table[symbol]&3)<<4);
PORTC|=(dot<<5);
}




void main( void )
{
DDRB=(1<<0)|(1<<1)| (1<<2)| (1<<3)| (1<<4)| (1<<5);//set as outputs tranzistor drivers
DDRC=(1<<2)|(1<<0);
PORTC=(1<<0);
int a,i,iold,iconv,deltai,v,vold,deltav,dot;
const long delay=20000;

while(1){
  deltav=abs((vold- adc_convert(1)));
  if(deltav>1){
  vold=adc_convert(1);
  }
v=(int)(vold*3.710982568);
dot=0;
if (v<1000){
  dot=1;
  v=v*10;
}
iconv=adc_average(7,100);
  deltai=abs(iold-iconv);
  if(deltai>1){
  iold=iconv;
  }
i=(int)(iold*0.551948051);


a = i % 10;
seven_segment(a, 0);
PORTB=(1<<0);
__delay_cycles(delay);

a = (i / 10) % 10;
seven_segment(a, 0);
PORTB=(1<<1);
__delay_cycles(delay);

a = i / 100;
seven_segment(a, 1);
PORTB=(1<<2);
__delay_cycles(delay);
 

a = (v / 10) % 10;
seven_segment(a, 0);
PORTB=(1<<3);
__delay_cycles(delay);

a = (v / 100) % 10;
seven_segment(a, !(dot));
PORTB=(1<<4);
__delay_cycles(delay);

a = v / 1000;
seven_segment(a, dot);
PORTB=(1<<5);
__delay_cycles(delay);
}    
  
  

}
